#include "read_count.h"

SEXP directionality(SEXP all, SEXP bin, SEXP span, SEXP first_bin, SEXP last_bin) try {

	if (!isInteger(span) || LENGTH(span)!=1) { throw std::runtime_error("span to compute directionality must be an integer scalar"); }
	const size_t sp=asInteger(span);

	// Getting the indices of the first and last bin on the chromosome.
	if (!isInteger(first_bin) || LENGTH(first_bin)!=1) { throw std::runtime_error("index of first bin must be an integer scalar"); }
	const int fbin=asInteger(first_bin);
	if (!isInteger(last_bin) || LENGTH(last_bin)!=1) { throw std::runtime_error("index of last bin must be an integer scalar"); }
	const int lbin=asInteger(last_bin);

	// Setting up the binning engine.
	binner engine(all, bin, fbin, lbin);
	const int nlibs=engine.get_nlibs();

	// Setting up the memory containers.
	const int nbins=lbin-fbin+1;
	int* curcounts=(int*)R_alloc(nlibs*nbins, sizeof(int)); 
	bool* ischanged=(bool*)R_alloc(nbins, sizeof(bool));
	for (int i=0; i<nbins; ++i) { ischanged[i]=false; }
	std::deque<int> waschanged, counts, anchors, targets;

	// Setting up the output vectors immediately.
	SEXP output=PROTECT(allocVector(VECSXP, 2));
try {
	SET_VECTOR_ELT(output, 0, allocMatrix(INTSXP, nbins, nlibs));
	int** downptrs=(int**)R_alloc(nlibs, sizeof(int*));
	SET_VECTOR_ELT(output, 1, allocMatrix(INTSXP, nbins, nlibs));
	int** upptrs=(int**)R_alloc(nlibs, sizeof(int*));
    if (nlibs) {
        downptrs[0]=INTEGER(VECTOR_ELT(output, 0));
        upptrs[0]=INTEGER(VECTOR_ELT(output, 1));
        for (int i=1; i<nlibs; ++i) {
            downptrs[i]=downptrs[i-1]+nbins;
            upptrs[i]=upptrs[i-1]+nbins;
        }
	    for (int i=0; i<nlibs; ++i) {
            std::fill(downptrs[i], downptrs[i]+nbins, 0);
            std::fill(upptrs[i], upptrs[i]+nbins, 0);
        }
    }

	// Other assorted sundries.
	size_t vecdex;
	int rowdex, curanchor;
	double current_average;
   	size_t diff;
    int lib;
    int * bpcounts;

	while (!engine.empty()) {
		engine.fill(curcounts, ischanged, waschanged);
		curanchor=engine.get_anchor() - fbin;
		for (vecdex=0; vecdex<waschanged.size(); ++vecdex) {
			rowdex=waschanged[vecdex];
	
			// Filling up the directionality indices.		
			diff=curanchor-rowdex;
			if (diff && diff <= sp) { 
                bpcounts=curcounts+rowdex*nlibs;
                for (lib=0; lib<nlibs; ++lib) {
                    const int& thiscount=bpcounts[lib];
                    downptrs[lib][curanchor]+=thiscount;
                    upptrs[lib][rowdex]+=thiscount;
                }
			} 

			// Resetting the ischanged vector for the next stretch of bin anchors.
			ischanged[waschanged[vecdex]]=false;
		}
		waschanged.clear();
	}
} catch (std::exception& e) { 
	UNPROTECT(1);
	throw;
}
	UNPROTECT(1);
	return output;
} catch (std::exception& e) {
	return mkString(e.what());
}

